import { IKeyValue, ITransformState, Nullable, Disposable, EventSubject } from '@univerjs/core';
import { IDragEvent, IMouseEvent, IPointerEvent, IWheelEvent } from './basics/i-events';
import { IObjectFullState, ITransformChangeState } from './basics/interfaces';
import { ITransformerConfig } from './basics/transformer-config';
import { IViewportInfo, Vector2 } from './basics/vector2';
import { UniverRenderingContext } from './context';
import { Engine } from './engine';
import { Layer } from './layer';
import { Scene } from './scene';
import { CURSOR_TYPE, RENDER_CLASS_TYPE } from './basics/const';
import { Transform } from './basics/transform';
export declare const BASE_OBJECT_ARRAY: string[];
export declare enum ObjectType {
    UNKNOWN = 0,
    RICH_TEXT = 1,
    SHAPE = 2,
    IMAGE = 3,
    RECT = 4,
    CIRCLE = 5,
    CHART = 6
}
export declare abstract class BaseObject extends Disposable {
    groupKey?: string;
    isInGroup: boolean;
    objectType: ObjectType;
    onTransformChange$: EventSubject<ITransformChangeState>;
    onPointerDown$: EventSubject<IMouseEvent | IPointerEvent>;
    onPointerMove$: EventSubject<IMouseEvent | IPointerEvent>;
    onPointerUp$: EventSubject<IMouseEvent | IPointerEvent>;
    onPointerOut$: EventSubject<IMouseEvent | IPointerEvent>;
    onPointerOver$: EventSubject<IMouseEvent | IPointerEvent>;
    onPointerLeave$: EventSubject<IMouseEvent | IPointerEvent>;
    onPointerEnter$: EventSubject<IMouseEvent | IPointerEvent>;
    onSingleClick$: EventSubject<IMouseEvent | IPointerEvent>;
    onClick$: EventSubject<IMouseEvent | IPointerEvent>;
    onDblclick$: EventSubject<IMouseEvent | IPointerEvent>;
    onTripleClick$: EventSubject<IMouseEvent | IPointerEvent>;
    onMouseWheel$: EventSubject<IWheelEvent>;
    onDragLeave$: EventSubject<IMouseEvent | IDragEvent>;
    onDragOver$: EventSubject<IMouseEvent | IDragEvent>;
    onDragEnter$: EventSubject<IMouseEvent | IDragEvent>;
    onDrop$: EventSubject<IMouseEvent | IDragEvent>;
    onIsAddedToParent$: EventSubject<any>;
    onDispose$: EventSubject<BaseObject>;
    protected _oKey: string;
    protected _dirty: boolean;
    protected _forceDirty: boolean;
    private _printable;
    private _top;
    private _topOrigin;
    private _left;
    private _leftOrigin;
    private _width;
    private _widthOrigin;
    private _height;
    private _heightOrigin;
    private _angle;
    private _scaleX;
    private _scaleY;
    private _skewX;
    private _skewY;
    private _flipX;
    private _flipY;
    private _strokeWidth;
    private _parent;
    private _zIndex;
    private _evented;
    private _visible;
    private _debounceParentDirty;
    private _transform;
    private _cursor;
    private _transformerConfig;
    private _forceRender;
    private _layer;
    constructor(key?: string);
    get transform(): Transform;
    transformForAngle(transform: Transform): Transform;
    get printable(): boolean;
    get topOrigin(): string | number;
    get leftOrigin(): string | number;
    get widthOrigin(): string | number;
    get heightOrigin(): string | number;
    get classType(): RENDER_CLASS_TYPE;
    get top(): number;
    get left(): number;
    get width(): number;
    get height(): number;
    get strokeWidth(): number;
    get angle(): number;
    get scaleX(): number;
    get scaleY(): number;
    get ancestorScaleX(): number;
    get ancestorScaleY(): number;
    get ancestorLeft(): any;
    get ancestorTop(): any;
    get ancestorTransform(): any;
    get ancestorGroup(): Nullable<BaseObject>;
    get skewX(): number;
    get skewY(): number;
    get flipX(): boolean;
    get flipY(): boolean;
    get parent(): any;
    get oKey(): string;
    get zIndex(): number;
    get evented(): boolean;
    get visible(): boolean;
    get debounceParentDirty(): boolean;
    get cursor(): CURSOR_TYPE;
    get layer(): Nullable<Layer>;
    set transform(trans: Transform);
    set zIndex(index: number);
    set parent(o: any);
    set evented(state: boolean);
    set debounceParentDirty(state: boolean);
    set cursor(val: CURSOR_TYPE);
    set layer(layer: Layer);
    protected set top(num: number | string);
    protected set left(num: number | string);
    protected set width(num: number | string);
    protected set height(num: number | string);
    protected set strokeWidth(width: number);
    protected set angle(angle: number);
    protected set scaleX(scaleX: number);
    protected set scaleY(scaleY: number);
    protected set skewX(skewX: number);
    protected set flipY(flipY: boolean);
    protected set flipX(flipX: boolean);
    protected set skewY(skewY: number);
    get transformerConfig(): ITransformerConfig;
    set transformerConfig(config: ITransformerConfig);
    get maxZIndex(): number;
    makeDirty(state?: boolean): this | undefined;
    makeForceDirty(state?: boolean): void;
    makeDirtyNoDebounce(state?: boolean): this;
    isDirty(): boolean;
    translate(x?: number | string, y?: number | string): this;
    resize(width?: number | string, height?: number | string): this;
    scale(scaleX?: number, scaleY?: number): this;
    skew(skewX?: number, skewY?: number): this;
    flip(flipX?: boolean, flipY?: boolean): this;
    /**
     * this[pKey] = option[pKey]
     * @param option
     */
    transformByState(option: IObjectFullState): this | undefined;
    isRender(bounds?: IViewportInfo): boolean | undefined;
    getParent(): any;
    getState(): ITransformState;
    hide(): void;
    show(): void;
    render(ctx: UniverRenderingContext, bounds: IViewportInfo): void;
    isHit(coord: Vector2): boolean;
    triggerPointerMove(evt: IPointerEvent | IMouseEvent): boolean;
    triggerPointerDown(evt: IPointerEvent | IMouseEvent): boolean;
    triggerPointerUp(evt: IPointerEvent | IMouseEvent): boolean;
    triggerSingleClick(evt: IPointerEvent | IMouseEvent): boolean;
    triggerClick(evt: IPointerEvent | IMouseEvent): boolean;
    triggerDblclick(evt: IPointerEvent | IMouseEvent): boolean;
    triggerTripleClick(evt: IPointerEvent | IMouseEvent): boolean;
    triggerMouseWheel(evt: IWheelEvent): boolean;
    triggerPointerOut(evt: IPointerEvent | IMouseEvent): boolean;
    triggerPointerLeave(evt: IPointerEvent | IMouseEvent): boolean;
    triggerPointerOver(evt: IPointerEvent | IMouseEvent): boolean;
    triggerPointerEnter(evt: IPointerEvent | IMouseEvent): boolean;
    triggerPointerCancel(evt: IPointerEvent): boolean;
    triggerDragLeave(evt: IDragEvent | IMouseEvent): boolean;
    triggerDragOver(evt: IDragEvent | IMouseEvent): boolean;
    triggerDragEnter(evt: IDragEvent | IMouseEvent): boolean;
    triggerDrop(evt: IDragEvent | IMouseEvent): boolean;
    dispose(): void;
    toJson(): IKeyValue;
    getScene(): Nullable<Scene>;
    resetCursor(): void;
    setCursor(val: CURSOR_TYPE): void;
    getEngine(): Nullable<Engine>;
    getObjects(): BaseObject[];
    getLayerIndex(): number;
    applyTransform(): void;
    removeTransform(): void;
    getInverseCoord(coord: Vector2): Vector2;
    protected _setTransForm(): void;
    private _makeDirtyMix;
}
